@@ -1,3 +1,21 @@
+/*
+ * libiio - Library for interfacing industrial I/O (IIO) devices
+ *
+ * Copyright (C) 2014-2016 Analog Devices, Inc.
+ * Author: Paul Cercueil <paul.cercueil@analog.com>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ */
+
 #include "debug.h"
 #include "iiod-client.h"
 #include "iio-lock.h"
@@ -195,7 +213,8 @@ int iiod_client_get_trigger(struct iiod_client *client, void *desc,
 	unsigned int name_len;
 	int ret;
 
-	snprintf(buf, sizeof(buf), "GETTRIG %s\r\n", iio_device_get_id(dev));
+	iio_snprintf(buf, sizeof(buf), "GETTRIG %s\r\n",
+			iio_device_get_id(dev));
 
 	iio_mutex_lock(client->lock);
 	ret = iiod_client_exec_command(client, desc, buf);
@@ -246,13 +265,14 @@ int iiod_client_set_trigger(struct iiod_client *client, void *desc,
 	char buf[1024];
 	int ret;
 
-	if (trigger)
-		snprintf(buf, sizeof(buf), "SETTRIG %s %s\r\n",
+	if (trigger) {
+		iio_snprintf(buf, sizeof(buf), "SETTRIG %s %s\r\n",
 				iio_device_get_id(dev),
 				iio_device_get_id(trigger));
-	else
-		snprintf(buf, sizeof(buf), "SETTRIG %s\r\n",
+	} else {
+		iio_snprintf(buf, sizeof(buf), "SETTRIG %s\r\n",
 				iio_device_get_id(dev));
+	}
 
 	iio_mutex_lock(client->lock);
 	ret = iiod_client_exec_command(client, desc, buf);
@@ -266,7 +286,7 @@ int iiod_client_set_kernel_buffers_count(struct iiod_client *client, void *desc,
 	int ret;
 	char buf[1024];
 
-	snprintf(buf, sizeof(buf), "SET %s BUFFERS_COUNT %u\r\n",
+	iio_snprintf(buf, sizeof(buf), "SET %s BUFFERS_COUNT %u\r\n",
 			iio_device_get_id(dev), nb_blocks);
 
 	iio_mutex_lock(client->lock);
@@ -281,7 +301,7 @@ int iiod_client_set_timeout(struct iiod_client *client,
 	int ret;
 	char buf[1024];
 
-	snprintf(buf, sizeof(buf), "TIMEOUT %u\r\n", timeout);
+	iio_snprintf(buf, sizeof(buf), "TIMEOUT %u\r\n", timeout);
 
 	iio_mutex_lock(client->lock);
 	ret = iiod_client_exec_command(client, desc, buf);
@@ -289,9 +309,31 @@ int iiod_client_set_timeout(struct iiod_client *client,
 	return ret;
 }
 
+static int iiod_client_discard(struct iiod_client *client, void *desc,
+		char *buf, size_t buf_len, size_t to_discard)
+{
+	do {
+		size_t read_len;
+		ssize_t ret;
+
+		if (to_discard > buf_len)
+			read_len = buf_len;
+		else
+			read_len = to_discard;
+
+		ret = iiod_client_read_all(client, desc, buf, read_len);
+		if (ret < 0)
+			return ret;
+
+		to_discard -= (size_t) ret;
+	} while (to_discard);
+
+	return 0;
+}
+
 ssize_t iiod_client_read_attr(struct iiod_client *client, void *desc,
 		const struct iio_device *dev, const struct iio_channel *chn,
-		const char *attr, char *dest, size_t len, bool is_debug)
+		const char *attr, char *dest, size_t len, enum iio_attr_type type)
 {
 	const char *id = iio_device_get_id(dev);
 	char buf[1024];
@@ -301,25 +343,46 @@ ssize_t iiod_client_read_attr(struct iiod_client *client, void *desc,
 		if (chn) {
 			if (!iio_channel_find_attr(chn, attr))
 				return -ENOENT;
-		} else if (is_debug) {
-			if (!iio_device_find_debug_attr(dev, attr))
-				return -ENOENT;
 		} else {
-			if (!iio_device_find_attr(dev, attr))
-				return -ENOENT;
+			switch (type) {
+				case IIO_ATTR_TYPE_DEVICE:
+					if (!iio_device_find_attr(dev, attr))
+						return -ENOENT;
+					break;
+				case IIO_ATTR_TYPE_DEBUG:
+					if (!iio_device_find_debug_attr(dev, attr))
+						return -ENOENT;
+					break;
+				case IIO_ATTR_TYPE_BUFFER:
+					if (!iio_device_find_buffer_attr(dev, attr))
+						return -ENOENT;
+					break;
+				default:
+					return -EINVAL;
+			}
 		}
 	}
 
-	if (chn)
-		snprintf(buf, sizeof(buf), "READ %s %s %s %s\r\n", id,
+	if (chn) {
+		iio_snprintf(buf, sizeof(buf), "READ %s %s %s %s\r\n", id,
 				iio_channel_is_output(chn) ? "OUTPUT" : "INPUT",
 				iio_channel_get_id(chn), attr ? attr : "");
-	else if (is_debug)
-		snprintf(buf, sizeof(buf), "READ %s DEBUG %s\r\n",
-				id, attr ? attr : "");
-	else
-		snprintf(buf, sizeof(buf), "READ %s %s\r\n",
-				id, attr ? attr : "");
+	} else {
+		switch (type) {
+			case IIO_ATTR_TYPE_DEVICE:
+				iio_snprintf(buf, sizeof(buf), "READ %s %s\r\n",
+						id, attr ? attr : "");
+				break;
+			case IIO_ATTR_TYPE_DEBUG:
+				iio_snprintf(buf, sizeof(buf), "READ %s DEBUG %s\r\n",
+						id, attr ? attr : "");
+				break;
+			case IIO_ATTR_TYPE_BUFFER:
+				iio_snprintf(buf, sizeof(buf), "READ %s BUFFER %s\r\n",
+						id, attr ? attr : "");
+				break;
+		}
+	}
 
 	iio_mutex_lock(client->lock);
 
@@ -328,6 +391,7 @@ ssize_t iiod_client_read_attr(struct iiod_client *client, void *desc,
 		goto out_unlock;
 
 	if ((size_t) ret + 1 > len) {
+		iiod_client_discard(client, desc, dest, len, ret + 1);
 		ret = -EIO;
 		goto out_unlock;
 	}
@@ -350,7 +414,7 @@ out_unlock:
 
 ssize_t iiod_client_write_attr(struct iiod_client *client, void *desc,
 		const struct iio_device *dev, const struct iio_channel *chn,
-		const char *attr, const char *src, size_t len, bool is_debug)
+		const char *attr, const char *src, size_t len, enum iio_attr_type type)
 {
 	struct iio_context_pdata *pdata = client->pdata;
 	const struct iiod_client_ops *ops = client->ops;
@@ -363,26 +427,47 @@ ssize_t iiod_client_write_attr(struct iiod_client *client, void *desc,
 		if (chn) {
 			if (!iio_channel_find_attr(chn, attr))
 				return -ENOENT;
-		} else if (is_debug) {
-			if (!iio_device_find_debug_attr(dev, attr))
-				return -ENOENT;
 		} else {
-			if (!iio_device_find_attr(dev, attr))
-				return -ENOENT;
+			switch (type) {
+				case IIO_ATTR_TYPE_DEVICE:
+					if (!iio_device_find_attr(dev, attr))
+						return -ENOENT;
+					break;
+				case IIO_ATTR_TYPE_DEBUG:
+					if (!iio_device_find_debug_attr(dev, attr))
+						return -ENOENT;
+					break;
+				case IIO_ATTR_TYPE_BUFFER:
+					if (!iio_device_find_buffer_attr(dev, attr))
+						return -ENOENT;
+					break;
+				default:
+					return -EINVAL;
+			}
 		}
 	}
 
-	if (chn)
-		snprintf(buf, sizeof(buf), "WRITE %s %s %s %s %lu\r\n", id,
+	if (chn) {
+		iio_snprintf(buf, sizeof(buf), "WRITE %s %s %s %s %lu\r\n", id,
 				iio_channel_is_output(chn) ? "OUTPUT" : "INPUT",
 				iio_channel_get_id(chn), attr ? attr : "",
 				(unsigned long) len);
-	else if (is_debug)
-		snprintf(buf, sizeof(buf), "WRITE %s DEBUG %s %lu\r\n",
-				id, attr ? attr : "", (unsigned long) len);
-	else
-		snprintf(buf, sizeof(buf), "WRITE %s %s %lu\r\n",
-				id, attr ? attr : "", (unsigned long) len);
+	} else {
+		switch (type) {
+			case IIO_ATTR_TYPE_DEVICE:
+				iio_snprintf(buf, sizeof(buf), "WRITE %s %s %lu\r\n",
+						id, attr ? attr : "", (unsigned long) len);
+				break;
+			case IIO_ATTR_TYPE_DEBUG:
+				iio_snprintf(buf, sizeof(buf), "WRITE %s DEBUG %s %lu\r\n",
+						id, attr ? attr : "", (unsigned long) len);
+				break;
+			case IIO_ATTR_TYPE_BUFFER:
+				iio_snprintf(buf, sizeof(buf), "WRITE %s BUFFER %s %lu\r\n",
+						id, attr ? attr : "", (unsigned long) len);
+				break;
+		}
+	}
 
 	iio_mutex_lock(client->lock);
 	ret = ops->write(pdata, desc, buf, strlen(buf));
@@ -448,13 +533,14 @@ int iiod_client_open_unlocked(struct iiod_client *client, void *desc,
 	char buf[1024], *ptr;
 	size_t i;
 
-	snprintf(buf, sizeof(buf), "OPEN %s %lu ",
+	iio_snprintf(buf, sizeof(buf), "OPEN %s %lu ",
 			iio_device_get_id(dev), (unsigned long) samples_count);
 	ptr = buf + strlen(buf);
 
-	for (i = dev->words; i > 0; i--, ptr += 8)
-		snprintf(ptr, (ptr - buf) + i * 8, "%08" PRIx32,
+	for (i = dev->words; i > 0; i--, ptr += 8) {
+		iio_snprintf(ptr, (ptr - buf) + i * 8, "%08" PRIx32,
 				dev->mask[i - 1]);
+	}
 
 	strcpy(ptr, cyclic ? " CYCLIC\r\n" : "\r\n");
 
@@ -466,7 +552,7 @@ int iiod_client_close_unlocked(struct iiod_client *client, void *desc,
 {
 	char buf[1024];
 
-	snprintf(buf, sizeof(buf), "CLOSE %s\r\n", iio_device_get_id(dev));
+	iio_snprintf(buf, sizeof(buf), "CLOSE %s\r\n", iio_device_get_id(dev));
 	return iiod_client_exec_command(client, desc, buf);
 }
 
@@ -516,7 +602,7 @@ ssize_t iiod_client_read_unlocked(struct iiod_client *client, void *desc,
 	if (!len || words != (nb_channels + 31) / 32)
 		return -EINVAL;
 
-	snprintf(buf, sizeof(buf), "READBUF %s %lu\r\n",
+	iio_snprintf(buf, sizeof(buf), "READBUF %s %lu\r\n",
 			iio_device_get_id(dev), (unsigned long) len);
 
 	ret = iiod_client_write_all(client, desc, buf, strlen(buf));
@@ -561,7 +647,7 @@ ssize_t iiod_client_write_unlocked(struct iiod_client *client, void *desc,
 	char buf[1024];
 	int val;
 
-	snprintf(buf, sizeof(buf), "WRITEBUF %s %lu\r\n",
+	iio_snprintf(buf, sizeof(buf), "WRITEBUF %s %lu\r\n",
 			dev->id, (unsigned long) len);
 
 	ret = iiod_client_write_all(client, desc, buf, strlen(buf));
